import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Dropdown, Grid, Input, Menu, MenuButton, MenuItem, Option, Select, Sheet, Typography, useTheme, Box, Divider, Tooltip, } from "@mui/joy";
import { debounce } from "@mui/material";
import { useCallback, useMemo, useState } from "react";
import { useTranslation } from "react-i18next";
import nerdUII18n from "../i18n";
import { formatLabel, genericIcons } from "./utils";
import { UnfoldMore } from "@mui/icons-material";
import { useDispatch } from "react-redux";
import { setSelectedPresets } from "../redux/nerdAdvancedSearch";
/**
 *
 * @param {SearchableProps} props Component props
 * @returns {ReactElement} Searchable
 */
export function Searchable({ index, definition, filters, onFiltersChange, isNew, }) {
    const dispatch = useDispatch();
    const { t } = useTranslation("components", { i18n: nerdUII18n });
    const filter = filters[index];
    const onFilterValueChange = useCallback((value) => {
        if (value != undefined) {
            const newFilters = [...filters];
            newFilters[index].value = value;
            onFiltersChange(newFilters);
        }
    }, [filters]);
    const onFilterComparisonChange = (comparison) => {
        if (comparison != undefined) {
            const newFilters = [...filters];
            newFilters[index].comparison = comparison;
            if (comparison === "isNotNull" || comparison === "isNull") {
                newFilters[index].value = "null";
            }
            onFiltersChange(newFilters);
        }
    };
    const handleRemoveField = () => {
        if (index >= 0) {
            const newFilters = [...filters];
            newFilters.splice(index, 1);
            onFiltersChange(newFilters);
            dispatch(setSelectedPresets([]));
        }
    };
    const debouncedChangeHandler = useMemo(() => debounce(onFilterValueChange, 300), []);
    const getInteractor = () => {
        const [value, setValue] = useState(typeof filter.value !== "boolean" ? filter.value.toString() : null);
        if (!definition)
            return _jsx(Typography, { children: filter.value });
        if (definition.renderInput)
            return definition.renderInput(filter.value, onFilterValueChange, filter.comparison, onFilterComparisonChange);
        switch (definition.type) {
            case "bool":
                return (_jsx(Box, { sx: { display: "flex", alignItems: "center" }, children: _jsx(Comparison, { type: definition.type, 
                        // eslint-disable-next-line i18next/no-literal-string
                        value: filter.value ? "eq" : "neq", postfix: formatLabel(definition.label).toLowerCase(), onChange: (v) => {
                            onFilterValueChange(v === "eq");
                        } }) }));
            case "int":
                return (_jsxs(Box, { sx: { display: "flex", alignItems: "center" }, children: [_jsx(Typography, { color: "neutral", level: "body-sm", children: formatLabel(definition.label) }), _jsx(Comparison, { type: definition.type, value: filter.comparison, onChange: (value) => {
                                onFilterComparisonChange(value);
                            } }), filter.comparison !== "isNotNull" &&
                            filter.comparison !== "isNull" ? (_jsx(Input, { sx: {
                                width: 100,
                            }, autoFocus: isNew, placeholder: "Type here...", variant: "plain", type: "number", size: "sm", defaultValue: `${filter.value}`, onChange: (e) => {
                                if (e.target.value) {
                                    debouncedChangeHandler(Number(e.target.value));
                                }
                                else
                                    debouncedChangeHandler("");
                            } })) : null] }));
            case "string":
                return (_jsxs(Box, { sx: { display: "flex", alignItems: "center" }, children: [_jsx(Typography, { color: "neutral", level: "body-sm", children: formatLabel(definition.label) }), _jsx(Comparison, { type: definition.type, value: filter.comparison, onChange: (value) => {
                                onFilterComparisonChange(value);
                            } }), filter.comparison !== "isNotNull" &&
                            filter.comparison !== "isNull" ? (_jsx(Input, { placeholder: t("nerdAdvancedSearch.type_here"), defaultValue: `${filter.value}`, sx: {
                                width: 100,
                            }, autoFocus: isNew, variant: "plain", size: "sm", onChange: (e) => {
                                debouncedChangeHandler(e.target.value);
                            } })) : null] }));
            case "date":
                return (_jsxs(Box, { sx: { display: "flex", alignItems: "center" }, children: [_jsx(Typography, { color: "neutral", level: "body-sm", children: formatLabel(definition.label) }), _jsx(Comparison, { type: definition.type, value: filter.comparison, onChange: (value) => {
                                onFilterComparisonChange(value);
                            } }), filter.comparison !== "isNotNull" &&
                            filter.comparison !== "isNull" ? (_jsx(Input, { placeholder: t("nerdAdvancedSearch.type_here"), sx: {
                                width: 120,
                            }, autoFocus: isNew, variant: "plain", size: "sm", defaultValue: `${filter.value}`, type: "date", onChange: (e) => {
                                onFilterValueChange(e.target.value);
                            } })) : null] }));
            case "singleSelect":
                return (_jsxs(Box, { sx: { display: "flex", alignItems: "center" }, children: [_jsx(Typography, { color: "neutral", level: "body-sm", children: formatLabel(definition.label) }), _jsx(Comparison, { type: definition.type, value: filter.comparison, onChange: (value) => {
                                onFilterComparisonChange(value);
                            } }), filter.comparison !== "isNotNull" &&
                            filter.comparison !== "isNull" ? (_jsx(Select, { size: "sm", sx: {
                                width: 150,
                            }, slotProps: {
                                listbox: {
                                    sx: {
                                        zIndex: (theme) => theme.zIndex.modal,
                                    },
                                },
                            }, autoFocus: isNew, placeholder: "Select an option", variant: "plain", value: value, onChange: (_e, v) => {
                                onFilterValueChange(v || undefined);
                                setValue(v);
                            }, children: definition.valueOptions.map((o) => (_jsx(Option, { value: o.value, children: o.option }, o.value))) })) : null] }));
            default:
                return _jsx("div", {});
        }
    };
    let faIcon = "";
    if (definition) {
        faIcon = genericIcons[definition.type];
        if (definition.faIcon) {
            faIcon = definition.faIcon;
        }
        if (definition.getFaIcon && filter.value) {
            faIcon = definition.getFaIcon(filter.value);
        }
    }
    return (_jsxs(Sheet, { sx: {
            borderRadius: "8px",
            display: "flex",
            alignItems: "center",
            p: 1,
            gap: 0.5,
            height: 34,
        }, color: "primary", variant: "outlined", children: [definition ? (_jsx(Typography, { level: "body-sm", sx: { display: "flex", gap: 1 }, children: _jsx("i", { className: faIcon }) })) : ([]), _jsx(Divider, { orientation: "vertical" }), _jsx(Box, { sx: { flexGrow: 1 }, children: getInteractor() }), _jsx(Tooltip, { title: t("nerdAdvancedSearch.remove_this_field"), size: "sm", children: _jsx("i", { style: { color: "inherit", cursor: "pointer" }, className: "fa-solid fa-xmark", onClick: handleRemoveField }) })] }));
}
/**
 *
 * @param {ComparisonProps} props Component props
 * @returns {ReactElement} Comparison
 */
function Comparison({ type, value, onChange, prefix, postfix, }) {
    const { t } = useTranslation("components", { i18n: nerdUII18n });
    const theme = useTheme();
    const config = {
        string: [
            { comparison: "like", title: "is like" },
            { comparison: "eq", title: "is" },
            { comparison: "neq", title: "is not" },
            { comparison: "isNotNull", title: "is not empty" },
            { comparison: "isNull", title: "is empty" },
        ],
        bool: [
            { comparison: "eq", title: "is" },
            { comparison: "neq", title: "is not" },
        ],
        date: [
            { comparison: "eq", title: "is same as" },
            { comparison: "neq", title: "is not" },
            { comparison: "gte", title: "is after" },
            { comparison: "lte", title: "is before" },
            { comparison: "isNotNull", title: "is not empty" },
            { comparison: "isNull", title: "is empty" },
        ],
        int: [
            { comparison: "eq", title: "is" },
            { comparison: "neq", title: "is not" },
            { comparison: "gt", title: "is greater than" },
            { comparison: "gte", title: "is greater or equal than" },
            { comparison: "lt", title: "is less than" },
            { comparison: "lte", title: "is less or equal than" },
            { comparison: "isNotNull", title: "is not empty" },
            { comparison: "isNull", title: "is empty" },
        ],
        singleSelect: [
            { comparison: "eq", title: "is" },
            { comparison: "neq", title: "is not" },
            { comparison: "isNotNull", title: "is not empty" },
            { comparison: "isNull", title: "is empty" },
        ],
    };
    const getOptions = () => {
        const JSX = [];
        for (const key in config[type]) {
            if (Object.prototype.hasOwnProperty.call(config[type], key)) {
                const e = config[type][key];
                JSX.push(_jsxs(MenuItem, { onClick: () => {
                        onChange(e.comparison);
                    }, children: [prefix, " ", e.title, " ", postfix] }, e.comparison));
            }
        }
        return JSX;
    };
    return (_jsxs(Dropdown, { children: [_jsx(Tooltip, { title: t("nerdAdvancedSearch.change_comparison"), size: "sm", children: _jsx(MenuButton, { variant: "plain", size: "sm", sx: {
                        px: 1,
                        ml: type !== "bool" ? 1 : 0,
                    }, children: _jsxs(Grid, { container: true, gap: 1, children: [prefix ? _jsx(Grid, { children: prefix }) : null, _jsx(Grid, { children: _jsx("b", { style: { color: theme.palette.primary[600] }, children: config[type].find((e) => e.comparison === value)?.title }) }), postfix ? _jsx(Grid, { children: postfix }) : null, _jsx(Grid, { children: _jsx(UnfoldMore, { style: { fontSize: 11 } }) })] }) }) }), _jsx(Menu, { sx: {
                    zIndex: (theme) => theme.zIndex.modal,
                }, children: getOptions() })] }));
}
