import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { Avatar, Badge, Box, Checkbox, Chip, ChipDelete, Divider, Grid, IconButton, List, ListItem, ListItemButton, ListItemDecorator, Sheet, Tooltip, } from "@mui/joy";
import { Drawer } from "@mui/material";
import { Popper, ClickAwayListener } from "@mui/base";
import { DateTime } from "luxon";
import { useCallback, useEffect, useMemo, useRef, useState, } from "react";
import { useDispatch } from "react-redux";
import { Link } from "react-router-dom";
import { useLocation } from "react-router-dom";
import { nerdHistoryOpenSelector, setNerdHistoryOpen, setNerdHistoryOpenKey, } from "../redux/nerdHistory";
import { useSelector } from "react-redux";
import { arrayToMap } from "@nerdjs/nerd-core";
import { useTranslation } from "react-i18next";
import { useCookie } from "../nerdUseCookie";
/**
 *
 * @param {NerdHistoryProps} props Props
 * @returns {ReactElement} NerdHistory
 */
export function NerdHistory(props) {
    const { historyKey, cookieDomain, openKey, anchor = "right", pathesToIgnore, appID, apps: _apps, } = props;
    const { t } = useTranslation();
    const location = useLocation();
    const [cookie, setCookie] = useCookie(historyKey, "[]", {
        domain: cookieDomain,
        path: "/",
    });
    const history = useMemo(() => {
        try {
            const _history = JSON.parse(cookie);
            if (Array.isArray(_history))
                return _history;
            else
                return [];
        }
        catch (error) {
            console.error(error);
            return [];
        }
    }, [cookie, historyKey]);
    const [selectMode, setSelectMode] = useState(false);
    const [selectedItems, setSelectedItems] = useState(new Set());
    const [allSelected, setAllSelected] = useState(false);
    const open = useSelector(nerdHistoryOpenSelector);
    const dispatch = useDispatch();
    const [openFilters, setOpenFilters] = useState(false);
    const filterChipRef = useRef(null);
    const [filters, setFilters] = useState(new Set());
    useEffect(() => {
        const newVal = history.filter((h) => {
            const date = DateTime.fromMillis(h.date);
            if (DateTime.now().diff(date, "days").days >= 7)
                return false;
            else
                return true;
        });
        setCookie(JSON.stringify(newVal));
    }, []);
    useEffect(() => {
        dispatch(setNerdHistoryOpenKey(openKey));
    }, [openKey]);
    useEffect(() => {
        const _open = localStorage.getItem(openKey);
        dispatch(setNerdHistoryOpen(_open == "true" ? true : false));
    }, []);
    useEffect(() => {
        let match = false;
        if (pathesToIgnore)
            for (let i = 0; i < pathesToIgnore.length; i++) {
                const rx = pathesToIgnore[i];
                match = rx.test(location.pathname);
                if (match)
                    break;
            }
        if (!match)
            setTimeout(() => {
                const index = history.findIndex((h) => h.endpoint === window.location.origin + location.pathname);
                if (index !== -1) {
                    const newHistory = [...history];
                    newHistory[index] = {
                        date: DateTime.now().toMillis(),
                        endpoint: window.location.origin + location.pathname,
                        label: document.title,
                        subtitle: location.state?.subtitle || "",
                        appID,
                        pinned: newHistory[index].pinned,
                    };
                    setCookie(JSON.stringify(newHistory));
                }
                else
                    setCookie(JSON.stringify([
                        ...history,
                        {
                            date: DateTime.now().toMillis(),
                            endpoint: window.location.origin + location.pathname,
                            label: document.title,
                            subtitle: location.state?.subtitle || "",
                            appID,
                            pinned: false,
                        },
                    ]));
            }, 100);
    }, [location]);
    const onClose = () => {
        dispatch(setNerdHistoryOpen(false));
    };
    const apps = useMemo(() => {
        return arrayToMap(_apps);
    }, [_apps]);
    const sortedApps = useMemo(() => {
        return [..._apps].sort((a, b) => a.name && b.name
            ? a.name?.localeCompare(b.name)
            : a.name
                ? 1
                : b.name
                    ? -1
                    : 0);
    }, [_apps]);
    const historyPinned = useMemo(() => {
        return filters.size > 0
            ? history
                .filter((h) => filters.has(h.appID) && h.pinned)
                .sort((a, b) => b.date - a.date)
            : history.filter((h) => h.pinned).sort((a, b) => b.date - a.date);
    }, [history, filters]);
    const historyUnPinned = useMemo(() => {
        return filters.size > 0
            ? history
                .filter((h) => filters.has(h.appID) && !h.pinned)
                .sort((a, b) => b.date - a.date)
            : history.filter((h) => !h.pinned).sort((a, b) => b.date - a.date);
    }, [history, filters]);
    const pinOrUnpinItem = useCallback((item) => {
        const index = history.findIndex((h) => h.endpoint === item.endpoint);
        if (index !== -1) {
            const newHistory = [...history];
            newHistory[index] = {
                ...newHistory[index],
                pinned: !newHistory[index].pinned,
            };
            setCookie(JSON.stringify(newHistory));
        }
    }, [history]);
    return (_jsx(_Fragment, { children: _jsx(Drawer, { open: open, onClose: onClose, variant: "persistent", sx: {
                width: 250,
                flexShrink: 0,
                "& .MuiDrawer-paper": {
                    width: 250,
                },
            }, anchor: anchor, children: _jsxs(Box, { sx: {
                    display: "flex",
                    gap: 1,
                    alignItems: "stretch",
                    flexDirection: "column",
                    mt: { xs: "140px", sm: "100px" },
                }, children: [_jsx(Sheet, { sx: {
                            p: 0.5,
                        }, children: _jsxs(Grid, { container: true, spacing: 1, alignItems: "start", children: [selectMode ? (_jsxs(_Fragment, { children: [_jsx(Grid, { children: _jsx(Chip, { disabled: history.length === 0, size: "sm", variant: "soft", color: "primary", onClick: () => {
                                                    if (!allSelected) {
                                                        setSelectedItems(new Set(history.map((h) => h.date)));
                                                    }
                                                    else
                                                        setSelectedItems(new Set());
                                                    setAllSelected((s) => !s);
                                                }, children: allSelected ? "Unselect All" : "Select All" }) }), _jsx(Grid, { children: _jsx(Chip, { size: "sm", variant: "soft", onClick: () => {
                                                    setSelectMode(false);
                                                    setAllSelected(false);
                                                    setSelectedItems(new Set());
                                                }, children: t("common:global.cancel") }) }), _jsx(Grid, { ml: "auto", children: _jsx(Tooltip, { title: "Clear History", children: _jsx(IconButton, { disabled: selectedItems.size === 0, size: "sm", color: "danger", variant: "soft", onClick: () => {
                                                        setCookie(JSON.stringify(history.filter((h) => !selectedItems.has(h.date))));
                                                        setSelectedItems(new Set());
                                                        setAllSelected(false);
                                                        setSelectMode(false);
                                                    }, children: _jsx("i", { className: "fa-solid fa-trash" }) }) }) })] })) : (_jsxs(_Fragment, { children: [_jsxs(Grid, { children: [_jsx(Badge, { badgeContent: filters.size, size: "sm", color: "danger", children: _jsx(Chip, { ref: filterChipRef, disabled: history.length === 0, size: "sm", variant: "soft", color: "primary", onClick: () => {
                                                            setOpenFilters(true);
                                                        }, endDecorator: filters.size > 0 ? (_jsx(ChipDelete, { onDelete: () => setFilters(new Set()) })) : undefined, children: "Filter" }) }), _jsx(Popper, { style: {
                                                        zIndex: 1200,
                                                    }, open: openFilters, anchorEl: filterChipRef?.current, children: _jsx(ClickAwayListener, { onClickAway: () => setOpenFilters(false), children: _jsx(Sheet, { sx: {
                                                                borderRadius: "md",
                                                                boxShadow: "lg",
                                                                py: 0.5,
                                                            }, children: _jsx(List, { children: sortedApps.map((app) => {
                                                                    const isSelected = filters.has(app.id);
                                                                    return (_jsx(ListItem, { children: _jsx(ListItemButton, { selected: isSelected, onClick: () => {
                                                                                setFilters((filters) => {
                                                                                    const newFilters = new Set(filters);
                                                                                    if (isSelected)
                                                                                        newFilters.delete(app.id);
                                                                                    else
                                                                                        newFilters.add(app.id);
                                                                                    return newFilters;
                                                                                });
                                                                            }, children: _jsxs(Box, { display: "flex", gap: 1, height: "100%", width: "100%", alignItems: "center", children: [app.icon ? (_jsx(Tooltip, { title: app.name || "", children: _jsx("img", { height: 20, width: 20, src: app.icon, alt: "App Icon" }) })) : null, app.name, _jsx(Box, { ml: "auto", width: 4, height: "100%", sx: {
                                                                                            backgroundColor: app.color,
                                                                                        } })] }) }) }, app.id));
                                                                }) }) }) }) })] }), _jsx(Grid, { ml: "auto", children: _jsx(Chip, { disabled: history.length === 0, size: "sm", variant: "soft", color: "primary", onClick: () => {
                                                    setSelectMode(true);
                                                }, children: "Select" }) })] })), _jsx(Grid, { children: _jsx(Tooltip, { title: "Close", children: _jsx(IconButton, { size: "sm", variant: "plain", onClick: onClose, children: _jsx("i", { className: "fa-solid fa-xmark" }) }) }) })] }) }), historyPinned.length > 0 ? (_jsxs(Box, { width: "100%", height: "100%", children: [_jsx(List, { size: "sm", children: historyPinned.map((h, i) => filters.size == 0 || filters.has(h.appID) ? (_jsxs(ListItem, { children: [selectMode ? (_jsx(ListItemDecorator, { children: _jsx(Checkbox, { size: "sm", checked: selectedItems.has(h.date) || false, onChange: (e) => {
                                                    const checked = e.target.checked;
                                                    setSelectedItems((state) => {
                                                        const newState = new Set(state);
                                                        if (checked)
                                                            newState.add(h.date);
                                                        else
                                                            newState.delete(h.date);
                                                        return newState;
                                                    });
                                                    if (!checked) {
                                                        setAllSelected(false);
                                                    }
                                                } }) })) : (_jsx(IconButton, { size: "sm", onClick: () => pinOrUnpinItem(h), color: "danger", children: _jsx("i", { className: "fa-solid fa-thumbtack" }) })), _jsx(Tooltip, { title: h.subtitle || "", children: _jsx(ListItemButton, { component: Link, to: h.endpoint, sx: {
                                                    textDecoration: "none !important",
                                                }, children: _jsxs(Box, { display: "flex", gap: 1, height: "100%", width: "100%", alignItems: "center", children: [apps[h.appID]?.icon ? (_jsx(Tooltip, { title: apps[h.appID]?.name || "", children: _jsx("img", { height: 20, width: 20, src: apps[h.appID]?.icon, alt: "App Icon" }) })) : null, h.label, _jsx(Box, { ml: "auto", width: 4, height: "100%", sx: { backgroundColor: apps[h.appID]?.color } })] }) }) })] }, i)) : null) }), _jsx(Divider, {})] })) : null, _jsx(Box, { width: "100%", height: "100%", children: historyUnPinned.length > 0 ? (_jsx(List, { size: "sm", children: historyUnPinned.map((h, i) => filters.size == 0 || filters.has(h.appID) ? (_jsxs(ListItem, { sx: {
                                    "& .pin-button": {
                                        visibility: "hidden",
                                    },
                                    "&:hover": {
                                        "& .pin-button": {
                                            visibility: "visible",
                                        },
                                    },
                                }, children: [selectMode ? (_jsx(ListItemDecorator, { children: _jsx(Checkbox, { size: "sm", checked: selectedItems.has(h.date) || false, onChange: (e) => {
                                                const checked = e.target.checked;
                                                setSelectedItems((state) => {
                                                    const newState = new Set(state);
                                                    if (checked)
                                                        newState.add(h.date);
                                                    else
                                                        newState.delete(h.date);
                                                    return newState;
                                                });
                                                if (!checked) {
                                                    setAllSelected(false);
                                                }
                                            } }) })) : (_jsx(IconButton, { size: "sm", className: "pin-button", onClick: () => pinOrUnpinItem(h), children: _jsx("i", { className: "fa-solid fa-thumbtack" }) })), _jsx(Tooltip, { title: h.subtitle || "", children: _jsx(ListItemButton, { component: Link, to: h.endpoint, sx: {
                                                textDecoration: "none !important",
                                            }, children: _jsxs(Box, { display: "flex", gap: 1, height: "100%", width: "100%", alignItems: "center", children: [apps[h.appID]?.icon ? (_jsx(Tooltip, { title: apps[h.appID]?.name || "", children: _jsx("img", { height: 20, width: 20, src: apps[h.appID]?.icon, alt: "App Icon" }) })) : null, h.label, _jsx(Box, { ml: "auto", width: 4, height: "100%", sx: { backgroundColor: apps[h.appID]?.color } })] }) }) })] }, i)) : null) })) : (_jsx(Box, { display: "flex", justifyContent: "center", width: "100%", height: "100%", children: _jsx(Avatar, { size: "lg", sx: {
                                    mt: 2,
                                }, children: _jsx("i", { className: "fa-regular fa-cactus" }) }) })) })] }) }) }));
}
